package com.snplog.backend.adapter;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.CabinDTO;
import com.snplog.backend.entity.Cabin;
import com.snplog.backend.repository.CabinRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.Cabin} to and from
 * {@link.com.snplog.backend.dto.CabinDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class CabinAdapter {
	
	
	/**
	 *  {@link.com.snplog.backend.adapter.MyUserAdapter} for the {@link.com.snplog.backend.entity.MyUser} class.
	 */
	@Autowired
	private MyUserAdapter myUserAdapter;
	
	/**
	 *  {@link.com.snplog.backend.adapter.CabinAreaAdapter} for the {@link.com.snplog.backend.entity.CabinArea} class.
	 */
	@Autowired
	private CabinAreaAdapter cabinAreaAdapter;
	
	/**
	 *  {@link.com.snplog.backend.repository.CabinRepository} for the {@link.com.snplog.backend.entity.Cabin} class.
	 */
	@Autowired
	private CabinRepository cabinRepository;

	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param cabin, to be transformed into DTO Object
	 * @return CabinDTO {@link.com.snplog.backend.dto.CabinDTO}
	 */
	public CabinDTO toDTO(Cabin cabin) {
		return new CabinDTO(
				cabin.getId(),
				myUserAdapter.toDTOFinal(cabin.getCabinOwner()),
				cabinAreaAdapter.toDTOFinal(cabin.getCabinArea()),
				cabin.getCabinName(),
				cabin.getLongitude(),
				cabin.getLatitude(),
				cabin.getCountry(),
				cabin.getRegion(),
				cabin.getMunicipality(),
				cabin.getStreetAdress(),
				cabin.getKid(),
				cabin.getGid(),
				cabin.getBid(),
				cabin.getCabinStatus(),
				cabin.getLastPlowed(),
				cabin.getCreated()
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param cabin, to be transformed into DTO Object
	 * @return CabinDTO {@link.com.snplog.backend.dto.CabinDTO}
	 */
	public CabinDTO toDTOFinal(Cabin cabin) {
		return new CabinDTO(
				cabin.getId(),
				null,
				null,
				cabin.getCabinName(),
				cabin.getLongitude(),
				cabin.getLatitude(),
				cabin.getCountry(),
				cabin.getRegion(),
				cabin.getMunicipality(),
				cabin.getStreetAdress(),
				cabin.getKid(),
				cabin.getGid(),
				cabin.getBid(),
				cabin.getCabinStatus(),
				cabin.getLastPlowed(),
				cabin.getCreated()
				);
		
	}
	
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param cabinDTO, to be transformed into entity
	 * @return Cabin {@link.com.snplog.backend.entity.Cabin}
	 */
	public Cabin fromDTO(CabinDTO cabinDTO) {
		Optional<Cabin> result = cabinRepository.findById(cabinDTO.getId());
		
		Cabin cabin = null;
		
		if(result.isPresent()) {
			cabin = result.get();
			cabin.setCabinName(cabinDTO.getCabinName());
			cabin.setKid(cabinDTO.getKid());
			cabin.setGid(cabinDTO.getGid());
			cabin.setBid(cabinDTO.getBid());
			cabin.setCabinStatus(cabinDTO.getCabinStatus());
			cabin.setLastPlowed(cabinDTO.getLastPlowed());
		} else {
			throw new RuntimeException("Did not find the cabin id - " + cabinDTO.getId());
		}
		
		return cabin;
	}
	

	
}
