package com.snplog.backend.adapter;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.ChangeDTO;
import com.snplog.backend.entity.Change;
import com.snplog.backend.repository.ChangeRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.Change} to and from
 * {@link.com.snplog.backend.dto.ChangeDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class ChangeAdapter {

	/**
	 *  {@link.com.snplog.backend.adapter.MyUserAdapter} for the {@link.com.snplog.backend.entity.MyUser} class.
	 */
	@Autowired
	private MyUserAdapter myUserAdapter;
	
	/**
	 *  {@link.com.snplog.backend.repository.ChangeRepository} for the {@link.com.snplog.backend.entity.Change} class.
	 */
	@Autowired
	private ChangeRepository changeRepository;
	
	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param change, to be transformed into DTO Object
	 * @return ChangeDTO {@link.com.snplog.backend.dto.ChangeDTO}
	 */
	public ChangeDTO toDTO(Change change) {
		return new ChangeDTO(
				change.getId(),
				myUserAdapter.toDTOFinal(change.getChangedBy()),
				change.getCreated(),
				change.getTableName(),
				change.getColumnName(),
				change.getRowNumber(),
				change.getOldValue(),
				change.getNewValue()
				);
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param change, to be transformed into DTO Object
	 * @return ChangeDTO {@link.com.snplog.backend.dto.ChangeDTO}
	 */
	public ChangeDTO toDTOFinal(Change change) {
		return new ChangeDTO(
				change.getId(),
				null,
				change.getCreated(),
				change.getTableName(),
				change.getColumnName(),
				change.getRowNumber(),
				change.getOldValue(),
				change.getNewValue()
				);
	}
	
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param changeDTO, to be transformed into entity
	 * @return Change {@link.com.snplog.backend.entity.Change}
	 */
	public Change fromDTO(ChangeDTO changeDTO) {
		Optional<Change> result = changeRepository.findById(changeDTO.getId());
		
		Change change = null;
		
		if(result.isPresent()) {
			change = result.get();
			change.setTableName(changeDTO.getTableName());
			change.setColumnName(changeDTO.getColumnName());
			change.setRowNumber(changeDTO.getRowNumber());
			change.setOldValue(changeDTO.getOldValue());
			change.setNewValue(changeDTO.getNewValue());
		} else {
			throw new RuntimeException("Did not find the change id - " + changeDTO.getId());
		}
		
		return change;
	}
	
}
