package com.snplog.backend.adapter;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.MessageDTO;
import com.snplog.backend.entity.Message;
import com.snplog.backend.repository.MessageRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.Message} to and from
 * {@link.com.snplog.backend.dto.MessageDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class MessageAdapter {

	/**
	 *  {@link.com.snplog.backend.adapter.MyUserAdapter} for the {@link.com.snplog.backend.entity.MyUser} class.
	 */
	@Autowired
	private MyUserAdapter myUserAdapter;
	
	/**
	 *  {@link.com.snplog.backend.adapter.MessageThreadAdapter} for the {@link.com.snplog.backend.entity.MessageThread} class.
	 */
	@Autowired
	private MessageThreadAdapter messageThreadAdapter;
	
	/**
	 *  {@link.com.snplog.backend.repository.MessageRepository} for the {@link.com.snplog.backend.entity.Message} class.
	 */
	@Autowired
	private MessageRepository messageRepository;
	
	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param cabin, to be transformed into DTO Object
	 * @return MessageDTO {@link.com.snplog.backend.dto.MessageDTO}
	 */
	public MessageDTO toDTO(Message message) {
		return new MessageDTO(
				message.getId(),
				myUserAdapter.toDTOFinal(message.getSender()),
				messageThreadAdapter.toDTOFinal(message.getMessageThread()),
				message.getMessageContent(),
				message.getDateCreated()
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param message, to be transformed into DTO Object
	 * @return MessageDTO {@link.com.snplog.backend.dto.MessageDTO}
	 */
	public MessageDTO toDTOFinal(Message message) {
		return new MessageDTO(
				message.getId(),
				null,
				null,
				message.getMessageContent(),
				message.getDateCreated()
				);
		
	}
	
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param MessageDTO, to be transformed into entity
	 * @return Message {@link.com.snplog.backend.entity.Message}
	 */
	public Message fromDTO(MessageDTO messageDTO) {
		Optional<Message> result = messageRepository.findById(messageDTO.getId());
		
		Message message = null;
		
		if(result.isPresent()) {
			message = result.get();
			message.setMessageContent(messageDTO.getMessageContent());
		} else {
			throw new RuntimeException("Did not find the message id - " + messageDTO.getId());
		}
		
		return message;
	}
	
}
