package com.snplog.backend.adapter;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.MessageThreadDTO;
import com.snplog.backend.dto.MyUserDTO;
import com.snplog.backend.entity.MessageThread;
import com.snplog.backend.entity.MyUser;
import com.snplog.backend.repository.MessageThreadRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.MessageThread} to and from
 * {@link.com.snplog.backend.dto.MessageThreadDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class MessageThreadAdapter {
	
	/**
	 *  {@link.com.snplog.backend.adapter.MyUserAdapter} for the {@link.com.snplog.backend.entity.MyUser} class.
	 */
	@Autowired
	private MyUserAdapter myUserAdapter;
	
	/**
	 *  {@link.com.snplog.backend.repository.MessageThreadRepository} for the {@link.com.snplog.backend.entity.MessageThread} class.
	 */
	@Autowired
	private MessageThreadRepository messageThreadRepository;

	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param messageThread, to be transformed into DTO Object
	 * @return MessageThreadDTO {@link.com.snplog.backend.dto.MessageThreadDTO}
	 */
	public MessageThreadDTO toDTO(MessageThread messageThread) {
		List<MyUserDTO> participants = new ArrayList<>();
		
		for (MyUser myUser : messageThread.getMyUsers()) {
			participants.add(myUserAdapter.toDTOFinal(myUser));
		}
		
		return new MessageThreadDTO(
				messageThread.getId(),
				messageThread.getDateCreated(),
				messageThread.isClosed(), 
				messageThread.getDateClosed(),
				participants
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param messageThread, to be transformed into DTO Object
	 * @return MessageThreadDTO {@link.com.snplog.backend.dto.MessageThreadDTO}
	 */
	public MessageThreadDTO toDTOFinal(MessageThread messageThread) {
		return new MessageThreadDTO(
				messageThread.getId(),
				messageThread.getDateCreated(),
				messageThread.isClosed(), 
				messageThread.getDateClosed(),
				null
				);
		
	}

	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param messageThreadDTO, to be transformed into entity
	 * @return MessageThread {@link.com.snplog.backend.entity.MessageThread}
	 */
	public MessageThread fromDTO(MessageThreadDTO messageThreadDTO) {
		Optional<MessageThread> result = messageThreadRepository.findById(messageThreadDTO.getId());
		
		MessageThread messageThread = null;
		
		if(result.isPresent()) {
			messageThread = result.get();
			messageThread.setClosed(messageThreadDTO.isClosed());
			messageThread.setDateClosed(messageThreadDTO.getDateClosed());
		} else {
			throw new RuntimeException("Did not find the messageThread id - " + messageThreadDTO.getId());
		}
		
		return messageThread;
	}

}
