package com.snplog.backend.adapter;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.CabinDTO;
import com.snplog.backend.dto.MyUserDTO;
import com.snplog.backend.entity.Cabin;
import com.snplog.backend.entity.MyUser;
import com.snplog.backend.entity.MyUserRole;
import com.snplog.backend.entity.Role;
import com.snplog.backend.repository.MyUserRepository;
import com.snplog.backend.repository.MyUserRoleRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.MyUser} to and from
 * {@link.com.snplog.backend.dto.MyUserDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class MyUserAdapter {
	
	/**
	 *  {@link.com.snplog.backend.adapter.CabinAdapter} for the {@link.com.snplog.backend.entity.Cabin} class.
	 */
	@Autowired
	private CabinAdapter cabinAdapter;
	
	/**
	 *  {@link.com.snplog.backend.repository.MyUserRepository} for the {@link.com.snplog.backend.entity.MyUser} class.
	 */
	@Autowired
	private MyUserRepository myUserRepository;
	
	@Autowired
	private MyUserRoleRepository myUserRoleRepository;

	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param myUser, to be transformed into DTO Object
	 * @return MyUserDTO {@link.com.snplog.backend.dto.MyUserDTO}
	 */
	public MyUserDTO toDTO(MyUser myUser) {
		List<CabinDTO> cabinsDTO = new ArrayList<>();
		List<Role> roles = new ArrayList<>();
		
		for (Cabin cabin : myUser.getCabins()) {
			cabinsDTO.add(cabinAdapter.toDTOFinal(cabin));
		}
		
		for(MyUserRole myUserRole : myUserRoleRepository.getMyUserRoleByMyUser(myUser.getId())) {
			roles.add(myUserRole.getRole());
		}
		
		return new MyUserDTO(
				myUser.getId(),
				myUser.getFirstName(),
				myUser.getLastName(),
				myUser.getEmail(),
				myUser.getPhoneNumber(),
				myUser.getStreetAdress(),
				myUser.getCity(),
				myUser.getCountry(),
				myUser.getDateTimeCreated(),
				roles,
				cabinsDTO
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param myUser, to be transformed into DTO Object
	 * @return MyUserDTO {@link.com.snplog.backend.dto.MyUserDTO}
	 */
	public MyUserDTO toDTOFinal(MyUser myUser) {
		return new MyUserDTO(
				myUser.getId(),
				myUser.getFirstName(),
				myUser.getLastName(),
				myUser.getEmail(),
				myUser.getPhoneNumber(),
				myUser.getStreetAdress(),
				myUser.getCity(),
				myUser.getCountry(),
				myUser.getDateTimeCreated(),
				null,
				null
				);
		
	}

	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param myUserDTO, to be transformed into entity
	 * @return MyUser {@link.com.snplog.backend.entity.MyUser}
	 */
	public MyUser fromDTO(MyUserDTO myUserDTO) {
		Optional<MyUser> result = myUserRepository.findById(myUserDTO.getId());
		
		MyUser myUser = null;
		
		if(result.isPresent()) {
			myUser = result.get();
			myUser.setFirstName(myUserDTO.getFirstName());
			myUser.setLastName(myUserDTO.getLastName());
			myUser.setEmail(myUserDTO.getEmail());
			myUser.setPhoneNumber(myUserDTO.getPhoneNumber());
			myUser.setStreetAdress(myUserDTO.getStreetAdress());
			myUser.setCity(myUserDTO.getCity());
			myUser.setCountry(myUserDTO.getCountry());
		} else {
			throw new RuntimeException("Did not find the user id - " + myUserDTO.getId());

		}
		
		return myUser;
	}
		
}
