package com.snplog.backend.adapter;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.PlowerPositionDTO;
import com.snplog.backend.entity.PlowerPosition;
import com.snplog.backend.repository.PlowerPositionRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.PlowerPosition} to and from
 * {@link.com.snplog.backend.dto.PlowerPositionDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class PlowerPositionAdapter {
	
	/**
	 *  {@link.com.snplog.backend.repository.PlowerPositionRepository} for the {@link.com.snplog.backend.entity.PlowerPosition} class.
	 */
	@Autowired
	private PlowerPositionRepository plowerPositionRepository;
	
	/**
	 *  {@link.com.snplog.backend.adapter.MyUserAdapter} for the {@link.com.snplog.backend.entity.MyUser} class.
	 */
	@Autowired
	private MyUserAdapter myUserAdapter;

	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param plowerPosition, to be transformed into DTO Object
	 * @return PlowerPositionDTO {@link.com.snplog.backend.dto.PlowerPositionDTO}
	 */
	public PlowerPositionDTO toDTO(PlowerPosition plowerPosition) {
		return new PlowerPositionDTO(
				plowerPosition.getId(),
				myUserAdapter.toDTOFinal(plowerPosition.getPlower()),
				plowerPosition.getDateTime(),
				plowerPosition.getLatitude(),
				plowerPosition.getLongitude()
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param plowerPosition, to be transformed into DTO Object
	 * @return PlowerPositionDTO {@link.com.snplog.backend.dto.PlowerPositionDTO}
	 */
	public PlowerPositionDTO toDTOFinal(PlowerPosition plowerPosition) {
		return new PlowerPositionDTO(
				plowerPosition.getId(),
				null,
				plowerPosition.getDateTime(),
				plowerPosition.getLatitude(),
				plowerPosition.getLongitude()
				);
		
	}
	
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param plowerPositionDTO, to be transformed into entity
	 * @return PlowerPosition {@link.com.snplog.backend.entity.PlowerPosition}
	 */
	public PlowerPosition fromDTO(PlowerPositionDTO plowerPositionDTO) {
		Optional<PlowerPosition> result = plowerPositionRepository.findById(plowerPositionDTO.getId());
		
		PlowerPosition plowerPosition = null;
		
		if(result.isPresent()) {
			plowerPosition = result.get();
		} else {
			throw new RuntimeException("Did not find the plowerPosition id - " + plowerPositionDTO.getId());
		}
		
		return plowerPosition;
	}
	
}
