package com.snplog.backend.adapter;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.PlowerRequestDTO;
import com.snplog.backend.entity.PlowerRequest;
import com.snplog.backend.repository.PlowerRequestRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.PlowerRequest} to and from
 * {@link.com.snplog.backend.dto.PlowerRequestDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class PlowerRequestAdapter {
	
	/**
	 *  {@link.com.snplog.backend.repository.PlowerRequestRepository} for the {@link.com.snplog.backend.entity.PlowerRequest} class.
	 */
	@Autowired
	private PlowerRequestRepository plowerRequestRepository;
	
	/**
	 *  {@link.com.snplog.backend.adapter.MyUserAdapter} for the {@link.com.snplog.backend.entity.MyUser} class.
	 */
	@Autowired
	private MyUserAdapter myUserAdapter;
	
	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param plowerRequest, to be transformed into DTO Object
	 * @return PlowerRequestDTO {@link.com.snplog.backend.dto.PlowerRequestDTO}
	 */
	public PlowerRequestDTO toDTO(PlowerRequest plowerRequest) {
		return new PlowerRequestDTO(
				plowerRequest.getId(),
				myUserAdapter.toDTOFinal(plowerRequest.getPlower()),
				plowerRequest.getDateRequested(),
				plowerRequest.getDateResponded(),
				plowerRequest.isStatus()
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param plowerRequest, to be transformed into DTO Object
	 * @return PlowerRequestDTO {@link.com.snplog.backend.dto.PlowerRequestDTO}
	 */
	public PlowerRequestDTO toDTOFinal(PlowerRequest plowerRequest) {
		return new PlowerRequestDTO(
				plowerRequest.getId(),
				null,
				plowerRequest.getDateRequested(),
				plowerRequest.getDateResponded(),
				plowerRequest.isStatus()
				);
		
	}
	
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param plowerRequestDTO, to be transformed into entity
	 * @return PlowerRequest {@link.com.snplog.backend.entity.PlowerRequest}
	 */
	public PlowerRequest fromDTO(PlowerRequestDTO plowerRequestDTO) {
		Optional<PlowerRequest> result = plowerRequestRepository.findById(plowerRequestDTO.getId());
		
		PlowerRequest plowerRequest = null;
		
		if(result.isPresent()) {
			plowerRequest = result.get();
			plowerRequest.setDateResponded(plowerRequestDTO.getDateResponded());
			plowerRequest.setStatus(plowerRequestDTO.isStatus());
		} else {
			throw new RuntimeException("Did not find the plowerRequest id - " + plowerRequestDTO.getId());
		}
		
		return plowerRequest;
	}
	
}
