package com.snplog.backend.adapter;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.PlowingRequestDTO;
import com.snplog.backend.entity.PlowingRequest;
import com.snplog.backend.repository.PlowingRequestRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.PlowingRequest} to and from
 * {@link.com.snplog.backend.dto.PlowingRequestDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class PlowingRequestAdapter {
	
	/**
	 *  {@link.com.snplog.backend.repository.PlowingRequestRepository} for the {@link.com.snplog.backend.entity.PlowingRequest} class.
	 */
	@Autowired
	private PlowingRequestRepository plowingRequestRepository;
	
	/**
	 *  {@link.com.snplog.backend.adapter.CabinAdapter} for the {@link.com.snplog.backend.entity.Cabin} class.
	 */
	@Autowired
	private CabinAdapter cabinAdapter;
	
	/**
	 *  {@link.com.snplog.backend.adapter.CabinAreaAdapter} for the {@link.com.snplog.backend.entity.CabinArea} class.
	 */
	@Autowired
	private CabinAreaAdapter cabinAreaAdapter;

	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param plowingRequest, to be transformed into DTO Object
	 * @return PlowingRequestDTO {@link.com.snplog.backend.dto.PlowingRequestDTO}
	 */
	public PlowingRequestDTO toDTO(PlowingRequest plowingRequest) {
		return new PlowingRequestDTO(
				plowingRequest.getId(),
				cabinAdapter.toDTOFinal(plowingRequest.getCabin()),
				cabinAreaAdapter.toDTOFinal(plowingRequest.getCabinArea()),
				plowingRequest.getDateOfArrival(),
				plowingRequest.getDateOfDeparture(),
				plowingRequest.getDateRequested(),
				plowingRequest.getDateCompleted(),
				plowingRequest.isCompleted(),
				plowingRequest.getDescription()
				);
		
	}	
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param plowingRequest, to be transformed into DTO Object
	 * @return PlowingRequestDTO {@link.com.snplog.backend.dto.PlowingRequestDTO}
	 */
	public PlowingRequestDTO toDTOFinal(PlowingRequest plowingRequest) {
		return new PlowingRequestDTO(
				plowingRequest.getId(),
				null,
				null,
				plowingRequest.getDateOfArrival(),
				plowingRequest.getDateOfDeparture(),
				plowingRequest.getDateRequested(),
				plowingRequest.getDateCompleted(),
				plowingRequest.isCompleted(),
				plowingRequest.getDescription()
				);
		
	}	
	
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param plowingRequestDTO, to be transformed into entity
	 * @return PlowingRequest {@link.com.snplog.backend.entity.PlowingRequest}
	 */
	public PlowingRequest fromDTO(PlowingRequestDTO plowingRequestDTO) {
		Optional<PlowingRequest> result = plowingRequestRepository.findById(plowingRequestDTO.getId());
		
		PlowingRequest plowingRequest = null;
		
		if(result.isPresent()) {
			plowingRequest = result.get();
			plowingRequest.setDateOfArrival(plowingRequestDTO.getDateOfArrival());
			plowingRequest.setDateOfDeparture(plowingRequestDTO.getDateOfDeparture());
			plowingRequest.setDateCompleted(plowingRequestDTO.getDateCompleted());
			plowingRequest.setCompleted(plowingRequestDTO.isCompleted());
			plowingRequest.setDescription(plowingRequestDTO.getDescription());
		} else {
			throw new RuntimeException("Did not find the messageThread id - " + plowingRequestDTO.getId());
		}
		
		return plowingRequest;
	}
	
}
