package com.snplog.backend.adapter;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.SnowStatisticDTO;
import com.snplog.backend.entity.SnowStatistic;
import com.snplog.backend.repository.SnowStatisticRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.SnowStatistic} to and from
 * {@link.com.snplog.backend.dto.SnowStatisticDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class SnowStatisticAdapter {
	
	/**
	 *  {@link.com.snplog.backend.repository.SnowStatisticRepository} for the {@link.com.snplog.backend.entity.SnowStatistic} class.
	 */
	@Autowired
	private SnowStatisticRepository snowStatisticRepository;
	
	/**
	 *  {@link.com.snplog.backend.adapter.WeatherStationAdapter} for the {@link.com.snplog.backend.entity.WeatherStation} class.
	 */
	@Autowired
	private WeatherStationAdapter weatherStationAdapter;

	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param snowStatistic, to be transformed into DTO Object
	 * @return SnowStatisticDTO {@link.com.snplog.backend.dto.SnowStatisticDTO}
	 */
	public SnowStatisticDTO toDTO(SnowStatistic snowStatistic) {
		return new SnowStatisticDTO(
				snowStatistic.getId(),
				weatherStationAdapter.toDTOFinal(snowStatistic.getWeatherStation()),
				snowStatistic.getCreated(),
				snowStatistic.getSurfaceSnowThickness()
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param snowStatistic, to be transformed into DTO Object
	 * @return SnowStatisticDTO {@link.com.snplog.backend.dto.SnowStatisticDTO}
	 */
	public SnowStatisticDTO toDTOFinal(SnowStatistic snowStatistic) {
		return new SnowStatisticDTO(
				snowStatistic.getId(),
				null,
				snowStatistic.getCreated(),
				snowStatistic.getSurfaceSnowThickness()
				);
		
	}
	
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param snowStatisticDTO, to be transformed into entity
	 * @return SnowStatistic {@link.com.snplog.backend.entity.SnowStatistic}
	 */
	public SnowStatistic fromDTO(SnowStatisticDTO snowStatisticDTO) {
		Optional<SnowStatistic> result = snowStatisticRepository.findById(snowStatisticDTO.getId());
		
		SnowStatistic snowStatistic = null;
		
		if(result.isPresent()) {
			snowStatistic = result.get();
		} else {
			throw new RuntimeException("Did not find the snowStatistic id - " + snowStatisticDTO.getId());
		}
		
		return snowStatistic;
	}
	
	
}
