package com.snplog.backend.adapter;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.CabinAreaDTO;
import com.snplog.backend.dto.SnowStatisticDTO;
import com.snplog.backend.dto.WeatherStationDTO;
import com.snplog.backend.entity.CabinArea;
import com.snplog.backend.entity.SnowStatistic;
import com.snplog.backend.entity.WeatherStation;
import com.snplog.backend.repository.WeatherStationRepository;

/**
 * Adapter to transform {@link.com.snplog.backend.entitiy.WeatherStation} to and from
 * {@link.com.snplog.backend.dto.WeatherStationDTO}.
 * This class is only referenced by the {@link.com.snplog.backend.services.misc.JpaDtoMapper},
 * and other adapter classes to link other objects.
 * 
 * @author Theodor Urtegård
 *
 */
@Service
public class WeatherStationAdapter {
	
	/**
	 *  {@link.com.snplog.backend.repository.WeatherStationRepository} for the {@link.com.snplog.backend.entity.WeatherStation} class.
	 */
	@Autowired
	private WeatherStationRepository weatherStationRepository;
	
	/**
	 *  {@link.com.snplog.backend.adapter.CabinAreaAdapter} for the {@link.com.snplog.backend.entity.CabinArea} class.
	 */
	@Autowired
	private CabinAreaAdapter cabinAreaAdapter;
	
	/**
	 *  {@link.com.snplog.backend.adapter.SnowStatisticAdapter} for the {@link.com.snplog.backend.entity.SnowStatistic} class.
	 */
	@Autowired
	private SnowStatisticAdapter snowStatisticAdapter;

	/**
	 * Creates a Data Transfer Object ready to send to an end user
	 * 
	 * @param weatherStation, to be transformed into DTO Object
	 * @return WeatherStationDTO {@link.com.snplog.backend.dto.WeatherStationDTO}
	 */
	public WeatherStationDTO toDTO(WeatherStation weatherStation) {
		List<CabinAreaDTO> cabinAreasDTO = new ArrayList<>();
		
		List<SnowStatisticDTO> snowStatisticsDTO = new ArrayList<>();
		
		for (CabinArea cabinArea : weatherStation.getCabinAreas()) {
			cabinAreasDTO.add(cabinAreaAdapter.toDTOFinal(cabinArea));
		}
		
		for (SnowStatistic snowStatistic : weatherStation.getSnowStatistics()) {
			snowStatisticsDTO.add(snowStatisticAdapter.toDTOFinal(snowStatistic));
		}
		
		return new WeatherStationDTO(
				weatherStation.getId(),
				weatherStation.getCreated(),
				weatherStation.getFrostId(),
				weatherStation.getLongitude(),
				weatherStation.getLatitude(),
				cabinAreasDTO,
				snowStatisticsDTO
				);
		
	}
	
	/**
	 * This function is needed to stop the infinite recursion in the JSON message sent to the end user
	 * 
	 * @param weatherStation, to be transformed into DTO Object
	 * @return WeatherStationDTO {@link.com.snplog.backend.dto.WeatherStationDTO}
	 */
	public WeatherStationDTO toDTOFinal(WeatherStation weatherStation) {
		return new WeatherStationDTO(
				weatherStation.getId(),
				weatherStation.getCreated(),
				weatherStation.getFrostId(),
				weatherStation.getLongitude(),
				weatherStation.getLatitude(),
				null,
				null
				);
	}
		
	/**
	 * Function to transform a Data Transfer Object to an Entity.
	 * Enabling the application to use jparepository calls (i.e. save, update, delete, etc..)
	 * 
	 * @param weatherStationDTO, to be transformed into entity
	 * @return WeatherStation {@link.com.snplog.backend.entity.WeatherStation}
	 */
	public WeatherStation fromDTO(WeatherStationDTO weatherStationDTO) {
		Optional<WeatherStation> result = weatherStationRepository.findById(weatherStationDTO.getId());
		
		WeatherStation weatherStation = null;
		
		if(result.isPresent()) {
			weatherStation = result.get();
		}
		
		return weatherStation;
	}

}
