package com.snplog.backend.rest;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.snplog.backend.dto.MyUserDTO;
import com.snplog.backend.entity.Change;
import com.snplog.backend.entity.MyUser;
import com.snplog.backend.models.SortedQueryRequest;
import com.snplog.backend.services.MyUserService;
import com.snplog.backend.services.misc.JpaDtoMapper;
import com.snplog.backend.services.misc.JwtUtil;

/**
 * 
 * @author Theodor Urtegård
 * 
 * Rest controller for {@link.com.snplog.backend.entity.MyUser} entity.
 *
 */
@RestController
@RequestMapping("/user")
public class MyUserRestController {

	@Autowired
	private MyUserService myUserService;
	
	@Autowired
	private JwtUtil jwtUtil;
	
	@Autowired
	private JpaDtoMapper jpaDtoMapper;
	
	@GetMapping("/users")
	public List<MyUserDTO> findAll() throws Exception {
		return myUserService.findAll();
	}
	
	/**
	 * 
	 * @param request
	 * @return ResponseEntity<?>, JSON Object
	 */
	@GetMapping("/myuser")
	public ResponseEntity<?> getMyUser(HttpServletRequest request){
		
		try {
			final String authorizationHeader = request.getHeader("Authorization");

			
			
			if(authorizationHeader != null && authorizationHeader.startsWith("Bearer ")) {
				String jwt = authorizationHeader.substring(7);
				String username = jwtUtil.extractUsername(jwt);
				
				MyUserDTO myUserDTO = myUserService.getMyUserDTOByEmail(username);

				return ResponseEntity.ok().header("Content-Type", "application/json; charset=utf-8").body(myUserDTO);
				
			} else {
				return ResponseEntity.badRequest().build();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return ResponseEntity.badRequest().build();
		}
	}
	
	/**
	 * 
	 * @param myUser {@link.com.snplog.backend.entity.MyUser}
	 * @return ResponseEntity<?>, JSON Object
	 */
	@PostMapping("/create")
	public ResponseEntity<?> createMyUser(@RequestBody MyUser myUser) {
		
		try {
			if(myUserService.checkIfUserExist(myUser.getEmail()) || myUser == null) {
				return ResponseEntity.badRequest().build();
			} else {
				myUserService.saveMyUser(myUser);
				return ResponseEntity.ok().header("Content-Type", "application/json; charset=utf-8").build();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return ResponseEntity.badRequest().build();
		}
	}
	
	/**
	 * 
	 * @param request
	 * @param myUserDTO {@link.com.snplog.backend.dto.MyUserDTO}
	 * @return ResponseEntity<?>, JSON Object
	 */
	@DeleteMapping("/delete/{id}")
	public ResponseEntity<?> deleteMyUser(HttpServletRequest request, @PathVariable(value="id") int id) {
		
		try {
			final String authorizationHeader = request.getHeader("Authorization");
			
			if(authorizationHeader != null && authorizationHeader.startsWith("Bearer" )) {
				
				String jwt = authorizationHeader.substring(7);
				

				if(jwtUtil.extractRoles(jwt).contains("admin") || jwtUtil.extractRoles(jwt).contains("plower")) {
					
					myUserService.deleteMyUserById(id);
					return ResponseEntity.ok().build();
				
				} 
				
			}
			
			return ResponseEntity.badRequest().build();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return ResponseEntity.badRequest().build();
		}
		
	}
	
	/**
	 * 
	 * @param request
	 * @param myUserDTO {@link.com.snplog.backend.dto.MyUserDTO}
	 * @return ResponseEntity<?>, JSON Object
	 */
	@PutMapping("/update")
	public ResponseEntity<?> updateMyUser(HttpServletRequest request, @RequestBody MyUserDTO myUserDTO) {
		
		try {
			final String authorizationHeader = request.getHeader("Authorization");
			
			if(authorizationHeader != null && authorizationHeader.startsWith("Bearer" )) {
				
				String jwt = authorizationHeader.substring(7);
				
				MyUser myUser = myUserService.getMyUserByEmail(jwtUtil.extractUsername(jwt));
				
				for(Change change : myUserDTO.getChanges()) {
					change.setChangedBy(myUser);
				}
				if(jwtUtil.extractRoles(jwt).contains("admin") || jwtUtil.extractRoles(jwt).contains("cabinOwner")) {
					
					myUserService.updateMyUser(myUserDTO);
					return ResponseEntity.ok().header("Content-Type", "application/json; charset=utf-8").build();
				} 
				
			}
			
			return ResponseEntity.badRequest().build();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return ResponseEntity.badRequest().build();
		}
	}
	
	/**
	 * 
	 * @param request
	 * @param sortedQueryRequest {@link.com.snplog.backend.models.SortedQueryRequest}
	 * @return ResponseEntity<?>, JSON Object
	 */
	@PostMapping("/sortedusers")
	public ResponseEntity<?> getSortedMyUsers(HttpServletRequest request, @RequestBody SortedQueryRequest sortedQueryRequest) {
		
		try {
			final String authorizationHeader = request.getHeader("Authorization");
			
			if(authorizationHeader != null && authorizationHeader.startsWith("Bearer" )) {
				
				String jwt = authorizationHeader.substring(7);
				
				List<String> roles = jwtUtil.extractRoles(jwt);
				
				List<MyUserDTO> myUserDTOs = myUserService.getMyUsersSortedBy(sortedQueryRequest);
				
				if(myUserDTOs != null) {
					return ResponseEntity.ok().header("Content-Type", "application/json; charset=utf-8").body(myUserDTOs);
				}
			}
			return ResponseEntity.badRequest().build();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return ResponseEntity.badRequest().build();
		}
	}
	
	/**
	 * 
	 * @param request
	 * @param firstName
	 * @return ResponseEntity<?>, JSON Object
	 */
	@GetMapping("/getusersbyfirstname/{firstName}")
	public ResponseEntity<?> getMyUsersByFirstNameSortedByFirstName(HttpServletRequest request, @PathVariable(value="firstName") String firstName) {
		try {
			final String authorizationHeader = request.getHeader("Authorization");
			
			if(authorizationHeader != null && authorizationHeader.startsWith("Bearer" )) {
				
				String jwt = authorizationHeader.substring(7);
				
				List<String> roles = jwtUtil.extractRoles(jwt);
				
				List<MyUserDTO> myUserDTOs = myUserService.getMyUsersByFirstNameSortedByFirstName(firstName);
				
				if(myUserDTOs != null) {
					return ResponseEntity.ok().header("Content-Type", "application/json; charset=utf-8").body(myUserDTOs);
				}
			}
			return ResponseEntity.badRequest().build();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return ResponseEntity.badRequest().build();
		}
	}
}
