package com.snplog.backend.services.implementations;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Timer;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import com.snplog.backend.dto.CabinAreaDTO;
import com.snplog.backend.dto.CabinDTO;
import com.snplog.backend.dto.MyUserDTO;
import com.snplog.backend.entity.Cabin;
import com.snplog.backend.entity.CabinArea;
import com.snplog.backend.entity.MyUser;
import com.snplog.backend.models.SortedQueryRequest;
import com.snplog.backend.repository.CabinRepository;
import com.snplog.backend.repository.ChangeRepository;
import com.snplog.backend.repository.MyUserRepository;
import com.snplog.backend.services.CabinService;
import com.snplog.backend.services.misc.CabinPositionService;
import com.snplog.backend.services.misc.JpaDtoMapper;
import com.snplog.backend.timertasks.CabinLocationHandler;

@Service
public class CabinServiceImpl implements CabinService {
	
	@Autowired
	private CabinRepository cabinRepository;
	
	@Autowired
	private JpaDtoMapper jpaDtoMapper;
	
	@Autowired
	private CabinPositionService cabinPositionService;
	
	@Autowired
	private MyUserRepository myUserRepository;
	
	@Autowired
	private ChangeRepository changeRepository;

	@Override
	public List<CabinDTO> findAll() {
		List<Cabin> cabins = cabinRepository.findAll();
		
		List<CabinDTO> cabinsDTO = new ArrayList<>();
		
		for (Cabin cabin : cabins ) {
			cabinsDTO.add(jpaDtoMapper.toDTO(cabin));
		}
		return cabinsDTO;
	}

	@Override
	public CabinDTO getCabin(int id) {
		Optional<Cabin> result = cabinRepository.findById(id);
		
		Cabin cabin = null;
		
		if(result.isPresent()) {
			cabin = result.get();
		} else {
			throw new RuntimeException("Did not find the cabin id - " + id);
		}
		
		return jpaDtoMapper.toDTO(cabin);
	}

	@Override
	public void saveCabin(Cabin cabin) {
		Optional<MyUser> result = myUserRepository.findById(cabin.getCabinOwner().getId());
		
		if(result.isPresent()) {
			cabin.setCabinOwner(result.get());
		}
		cabinRepository.save(cabin);
	}
	

	@Override
	public void deleteCabin(CabinDTO cabinDTO) {
		cabinRepository.deleteById(cabinDTO.getId());
	}

	@Override
	public void updateCabin(CabinDTO cabinDTO) {
		Optional<Cabin> result = cabinRepository.findById(cabinDTO.getId());
		
		Cabin cabin = null;
		
		if(result.isPresent()) {
			
			if(cabinDTO.isEdited()) {
				cabin = jpaDtoMapper.fromDTO(cabinDTO);
				
				cabinRepository.save(cabin);
				
				changeRepository.saveAll(cabinDTO.getChanges());
			}
		} else {
			throw new RuntimeException("Did not find the cabin id - " + cabinDTO.getId());
		}
		
	}
	
	@Override
	public void addLocation(Cabin cabin) {
		Timer timer = new Timer();
		timer.schedule(new CabinLocationHandler(cabin, cabinRepository, cabinPositionService), 10000);
	}
	
	@Override
	public List<CabinDTO> getCabinsByMyUser(int id){
		Optional<MyUser> result = myUserRepository.findById(id);
		
		MyUser myUser = null;
		
		if(result.isPresent()) {
			myUser = result.get();
			
			List<CabinDTO> cabins = new ArrayList<>();
			
			if(myUser.getCabins() != null) {
				for(Cabin cabin : myUser.getCabins()) {
					cabins.add(jpaDtoMapper.toDTO(cabin));
				}
				return cabins;
			}
		}
		return null;
	}
	
	@Override
	public List<CabinDTO> getCabinsSortedBy(SortedQueryRequest sortedQueryRequest) {
		
		List<Cabin> cabins = null;
		
		if(sortedQueryRequest.getOrder().contains("ASC")) {
			Pageable paging = PageRequest.of(
						sortedQueryRequest.getOffset(),
						sortedQueryRequest.getRowCount(),
						Sort.by(sortedQueryRequest.getColumn()).ascending()
					);
			
			cabins = cabinRepository.findAll(paging).toList();

		} else if (sortedQueryRequest.getOrder().contains("DESC")) {
			Pageable paging = PageRequest.of(
					sortedQueryRequest.getOffset(),
					sortedQueryRequest.getRowCount(),
					Sort.by(sortedQueryRequest.getColumn()).descending()
				);
		
		cabins = cabinRepository.findAll(paging).toList();
		}
		
		if(cabins != null) {
			List<CabinDTO> cabinDTOs = new ArrayList<>();
			
			for(Cabin cabin : cabins) {
				cabinDTOs.add(jpaDtoMapper.toDTO(cabin));
			}
			
			return cabinDTOs;
		}
		
		return null;
	}
	
	@Override
	public List<CabinDTO> getCabinsByCabinNameSortedByCabinName(String name){
		List<CabinDTO> result = new ArrayList<>();
		List<Cabin> cabins = cabinRepository.findByCabinNameOrderByCabinName(name);
		for(Cabin cabin : cabins) {
			result.add(jpaDtoMapper.toDTO(cabin));
		}
		return result;
	}
	
	@Override
	public void deleteCabinById(int id) {
		cabinRepository.deleteById(id);
	}
	
	@Override
	public Cabin getCabinById(int id) {
		Optional<Cabin> result = cabinRepository.findById(id);
		
		if(result.isPresent()) {
			return result.get();
		}
		
		return null;
	}

}
