package com.snplog.backend.services.misc;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;

import com.snplog.backend.entity.Role;
import com.snplog.backend.misc.MyUserPrincipal;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

@Service
public class JwtUtil {
	
	private static final int admin_expiration = 1000 * 60 * 60 * 8;
	
	private static final int cabin_owner_expiration = 1000 * 60 * 60 * 24 * 7;
	
	private static final int plower_expiration = 1000 * 60 * 60 * 8;
	
	private static final int user_expiration = 1000 * 60 * 60 * 24 * 7;

	@Value("$(spring.datasource.sercret-key)")
	private String SECRET_KEY;
	
	public String extractUsername(String token) {
		return extractClaim(token, Claims::getSubject);
	}
	
	@SuppressWarnings("unchecked")
	public List<String> extractRoles(String token) {
		final Claims claims = extractAllClaims(token);
		return (List<String>) claims.get("Roles");
	}
	
	public Date extractExpiration(String token) {
		return extractClaim(token, Claims::getExpiration);
	}
	
	public <T> T extractClaim(String token, Function<Claims, T> claimsResolver) {
		final Claims claims = extractAllClaims(token);
		return claimsResolver.apply(claims);
	}
	
	
	private Claims extractAllClaims(String token) {
		return Jwts.parser().setSigningKey(SECRET_KEY).parseClaimsJws(token).getBody();
	}
	
	public boolean isTokenExpired(String token) {
		return extractExpiration(token).before(new Date());
	}
	
	public String generateToken(MyUserPrincipal myUserPrincipal) {
		Map<String, Object> claims = new HashMap<>();
		return createToken(claims, myUserPrincipal.getUsername(), myUserPrincipal.getRoles(), generateExpiration(myUserPrincipal));
	}
	
	private int generateExpiration(MyUserPrincipal myUserPrincipal) {
		List<String> roles = myUserPrincipal.getRoles();
		if(roles.contains("cabinOwner")) {
			return cabin_owner_expiration;
		} else if(roles.contains("plower")) {
			return plower_expiration;
		} else if(roles.contains("admin")) {
			return admin_expiration;
		} else {
			return user_expiration;
		}
	}
	
	private String createToken(Map<String, Object> claims, String subject, List<String> roles, int expiration) {
		return Jwts.builder().setClaims(claims).setSubject(subject)
				.claim("Roles", roles)
				.setIssuedAt(new Date(System.currentTimeMillis()))
				.setExpiration(new Date(System.currentTimeMillis() + expiration))
				.signWith(SignatureAlgorithm.HS256, SECRET_KEY).compact();
	}
	
	public boolean validateToken(String token, UserDetails userDetails) {
		final String username = extractUsername(token);
		return (username.equals(userDetails.getUsername()) && !isTokenExpired(token));
	}
	
}
