import 'dart:async';

import 'package:flutter/material.dart';
import 'package:geolocator/geolocator.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:snow_plow/map/plower_cabin_info.dart';
import 'package:snow_plow/models/CabinDTO.dart';
import 'package:snow_plow/models/PlowingRequestDTO.dart';
import 'package:snow_plow/models/RouteRequest.dart';
import 'package:snow_plow/globals.dart' as globals;
import 'package:snow_plow/services/RouteService.dart';
import 'package:snow_plow/widgets/main_drawer.dart';

class PlowerMapView extends StatefulWidget {
  static const routeName = "/map";

  @override
  _PlowerMapView createState() => _PlowerMapView();
}

class _PlowerMapView extends State<PlowerMapView> {
  Completer<GoogleMapController> _controller = Completer();
  RouteService routeService = new RouteService();

  bool _visibleCabinInfo = false;

  Map<String, Marker> _posMarkers = {};

  List<LatLng> _coordinates;
  Set<Polyline> _polylines = {};

  PlowerCabinInfo _cabinInfoWindow;

  static const CameraPosition _initialCam = CameraPosition(
    target: LatLng(62.477778, 6.190278),
    zoom: 12,
  );

  void _onMapCreation(GoogleMapController controller) {
    _controller.complete(controller);
    _startTimer();
  }

  void _showCabinInfo(CabinDTO cabin) {
    setState(() {
      _cabinInfoWindow = new PlowerCabinInfo(cabin: cabin);
      _visibleCabinInfo = true;
    });
  }

  void _hideCabinInfo() {
    setState(() {
      _visibleCabinInfo = false;
    });
  }

  bool _isVisibleCabinInfo() {
    return _visibleCabinInfo;
  }

  void _startTimer() {
    const duration = const Duration(milliseconds: 1000);
    Timer.periodic(duration, (timer) {
      _updateCurrentPosition();
    });
  }

  void _updateCurrentPosition() async {
    var currentLocation = await Geolocator().getCurrentPosition(
        desiredAccuracy: LocationAccuracy.bestForNavigation);
    setState(() {
      final currentPosMarker = Marker(
        icon: BitmapDescriptor.defaultMarkerWithHue(BitmapDescriptor.hueCyan),
        markerId: MarkerId("currentPos"),
        position: LatLng(currentLocation.latitude, currentLocation.longitude),
      );
      _posMarkers["Current position"] = currentPosMarker;
    });
  }

  void _getDirections() {
    DateTime start = new DateTime.now();
    routeService
        .fetchRoute(new RouteRequest(
            dateOfArrival: start,
            dateOfDeparture:
                new DateTime(start.year + 1, start.month, start.day + 7),
            cabinAreaId: globals.selectedCabinArea.id,
            startLocation: "62.47,6.14",
            endLocation: "62.47,6.14"))
        .then((value) {
      if (value != null) {
        setState(() {
          _coordinates = value;
          Polyline polyline = Polyline(
              polylineId: PolylineId("Polyline"),
              color: Colors.blue,
              points: _coordinates,
              width: 2);
          _polylines.add(polyline);
          _setMapPins();
        });
      }
    });
  }

  void _setMapPins() {
    setState(() {
      _posMarkers["start/end"] = Marker(
          markerId: MarkerId("start/end"), position: LatLng(62.47, 6.14));
      for (PlowingRequestDTO pr in globals.plowingRequests) {
        _posMarkers[pr.cabin.cabinName] = Marker(
          markerId: MarkerId(pr.cabin.cabinName),
          position: LatLng(pr.cabin.latitude, pr.cabin.longitude),
          onTap: () {
            _showCabinInfo(pr.cabin);
          },
        );
      }
    });
  }

  @override
  void initState() {
    _getDirections();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      drawer: MainDrawer(),
      appBar: AppBar(
        title: Text("Kart"),
      ),
      body: Stack(
        children: <Widget>[
          GoogleMap(
            mapType: MapType.normal,
            initialCameraPosition: _initialCam,
            onMapCreated: _onMapCreation,
            markers: _posMarkers.values.toSet(),
            polylines: _polylines,
          ),
          _isVisibleCabinInfo()
              ? Container(
                  padding: EdgeInsets.fromLTRB(10, 10, 10, 10),
                  child: Align(
                    alignment: Alignment.bottomCenter,
                    child: _cabinInfoWindow,
                  ),
                )
              : Container(),
          _isVisibleCabinInfo()
          ? Container(
            padding: EdgeInsets.fromLTRB(0, 10, 15, 15),
            child: Align(
              alignment: Alignment.bottomRight,
              child: FloatingActionButton(
                backgroundColor: Theme.of(context).primaryColor,
                foregroundColor: Colors.white,
                onPressed: _hideCabinInfo,
                child: const Icon(Icons.close),
              ),
            ),
          )
          : Container(),
        ],
      ),
    );
  }
}